package net.wanji.opt.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.framework.Constants;
import net.wanji.opt.dto.CrossDataRealtimeDTO;
import net.wanji.opt.po.CrossDataRealtimePO;
import net.wanji.opt.service.CrossOptimizeService;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hfx
 * @date 2023/1/16 16:39
 * @desc RealtimeOptimizeTask
 */

@Component
@Slf4j
public class CrossMonitorTask {

    @Resource
    CrossOptimizeService crossOptimizeService;

    /**
     * 路口实时监测
     */
    @Scheduled(cron = "* 0/1 * * * ? ")
    public void realtimeMonitor() {

        log.info("路口实时监测任务...");
        // 获取路口实时监测数据
        List<CrossDataRealtimePO> crossDataRealtimePOList = listCrossDataRealtime();
        log.info("路口实时数据：{}条", crossDataRealtimePOList.size());
        // 测试时无数据暂时注释
//        if(crossDataRealtimePOList.isEmpty()) {
//            return ;
//        }

        // 获取所有信控路口异常数据（失衡/拥堵/溢出/死锁）
        List<CrossDataRealtimeDTO> abnormalCrossList = listAbnormalCross(crossDataRealtimePOList);
        log.info("加载异常路口：{}条", abnormalCrossList.size());
//        if(abnormalCrossList.isEmpty()) {
//            return ;
//        }

        // 调用路口实时优化
        crossOptimizeService.realtimeOptimize(abnormalCrossList);

    }

    /**
     * 获取异常路口数据
     * @return
     */
    public List<CrossDataRealtimeDTO> listAbnormalCross(List<CrossDataRealtimePO> crossDataRealtimePOList) {

        List<CrossDataRealtimeDTO> dtoList = new ArrayList<>();
        CrossDataRealtimeDTO dto = null;
        int status = 0;
        for (CrossDataRealtimePO po: crossDataRealtimePOList) {

            // 获取失衡、拥堵、溢出路口数据
            if(Constants.SystemParam.NO.equals(po.getIsSpillover()) || Constants.SystemParam.NO.equals(po.getIsCongestion()) || Constants.SystemParam.NO.equals(po.getIsUnbalance())) {
                continue;
            }

            if(Constants.SystemParam.YES.equals(po.getIsSpillover())) {         // 溢出
                status = CrossStatusEnum.SPILLOVER.getCode();
            } else if(Constants.SystemParam.YES.equals(po.getIsCongestion())) { // 拥堵
                status = CrossStatusEnum.CONGESTION.getCode();
            } else if(Constants.SystemParam.YES.equals(po.getIsUnbalance())) {  // 失衡
                status = CrossStatusEnum.UNBALANCE.getCode();
            }
            dto = new CrossDataRealtimeDTO();
            dto.setCrossId(po.getCrossId());
            dto.setStatus(status);
            dto.setDirList(null);
            dto.setTurnList(null);
            dtoList.add(dto);
        }
        return dtoList;
    }

    /**
     * 获取路口实时监测数据
     * （暂定从数据中心kafka获取）
     * @return
     */
    public List<CrossDataRealtimePO> listCrossDataRealtime() {



        return new ArrayList<>();
    }

}